import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;
import safehome.device.*;
/*
 * PROJECT : SafeHome
 * FILE : MainDemo.java
 * PURPOSE : To demonstrate SafeHome hardware devices to students
 * AUTHORSHIP : TA
 * REVISION HISTORY
 * 		2007/05/23 [TA] : merge several code to make this file
 * 		2007/05/24 [TA] : removed CameraView.poll(), CameraView.repaint() substitute removed method
 * 		2007/05/28 [TA] : modified CameraView.paint()
 * KNOWN BUG
 * 		In CameraView class,
 * 		CameraView.repaint() is accessed concurrently by own thread and by user's command
 * 		This code has regardlessness of whether Component.repaint() is thread-safe or not.
 * 		But during several test, there was no error of concurrency
 */


/*
 * CLASS NAME : MainDemo 
 * DESCRIPTION
 * 		A main dialog of whole demo program
 * 		It shows three buttons to enter three test cases 
 * 		 - Control panel test
 * 		 - Sensor test
 * 		 - Camera test
 * PRECONDITION : The program is executed initially
 * PARAMETER : none
 * RETURN VALUE : none
 * USAGE : click on the button
 * EXAMPLE : click the control panel test button to pop up control panel test dialog
 * POSTCONDITION : any one dialog of the test cases are pop up
 * NOTES : Click any button to proceed exact test case
 */  
public class MainDemo extends JFrame implements ActionListener 
{
	// instance of three test cases
	ControlPanel controlPanel = new ControlPanel();
	SensorTest sensorTest = new SensorTest();
	CameraTest cameraTest = new CameraTest();
	
	
	/*
	 * NAME : main 
	 * DESCRIPTION
	 * 		A main
	 * 		Show MainDemo dialog
	 * PRECONDITION : The program is executed initially
	 * PARAMETER : args - String[] but not used
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE : none
	 * POSTCONDITION : The program terminated
	 * NOTES : none
	 */  
	public static void main(String[] args)
	{
		MainDemo safeHome = new MainDemo();
		safeHome.setVisible(true);
	}
	
	/*
	 * NAME : MainDemo (constructor) 
	 * DESCRIPTION
	 * 		make layout of main page of this demo program
	 * 		place three buttons - "Control Panel", "sensor", "Camera"
	 * PRECONDITION : The main instantiates MainDemo class
	 * PARAMETER : none
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE : none
	 * POSTCONDITION : The layout of this dialog is made
	 * NOTES : none
	 */  
	public MainDemo() 
	{
		super("Hardware Device Demo");
		setSize(400, 215);
		getContentPane().setLayout(null);
		setResizable(false);

		// Terminate the program when the MainDemo dialog is closed
		addWindowListener
		(
				new WindowAdapter()
				{
					public void windowClosing(WindowEvent e)
					{
						System.exit(0);
					}
				}
		);
		
		// make buttons
		JButton ControlPanel = new JButton("Control Panel");
		ControlPanel.addActionListener(this);
		
		JButton SensorTest = new JButton("Sensor");
		SensorTest.addActionListener(this);
		
		JButton Camera = new JButton("Camera");
		Camera.addActionListener(this);
		
		// attach buttons to dialog
		JPanel displayPanel = new JPanel(new GridLayout(3, 1));
		displayPanel.add(ControlPanel);
		displayPanel.add(SensorTest);
		displayPanel.add(Camera);
		displayPanel.setBounds(15, 15, 360, 150);
		getContentPane().add(displayPanel);
	}

	/*
	 * NAME : actionPerformed
	 * DESCRIPTION
	 * 		This function is the action listener(call back function) of three buttons
	 * 		Each button pops up its test dialog
	 * PRECONDITION : User click on any button
	 * PARAMETER : event - ActionEvent that carrying what button was pressed
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE
	 * 		If Control Panel button is clicked by user, 
	 * 		the parameter event.getActionCommand() equals "Control Panel"
	 * POSTCONDITION : any one dialog of the test cases are pop up
	 * NOTES : none
	 */  
	public void actionPerformed(ActionEvent event)
	{
		if((event.getActionCommand().equals("Control Panel")))
		{
			controlPanel.setVisible(true);
		}
		else if((event.getActionCommand().equals("Sensor")))
		{
			sensorTest.setVisible(true);
		}
		else if((event.getActionCommand().equals("Camera")))
		{
			cameraTest.setVisible(true);
		}
	}
	
	/*
	 * meaningless member variable in this application
	 * prevent java compiler warning message
	 */
	static final long serialVersionUID = 1876534;
}



/*
 * CLASS NAME : ControlPanel 
 * DESCRIPTION
 * 		A simple implementation of the abstract class
 * 		All implemented abstract methods has small functionality to show it is working well
 * 		All APIs that provided from super class is tested by buttons
 * PRECONDITION : From main dialog, "Control Panel" button was chosen
 * PARAMETER : none
 * RETURN VALUE : none
 * USAGE : click the numeric buttons
 * EXAMPLE : click the button 1 to test its functionality
 * POSTCONDITION : none
 * NOTES : If possible, click all buttons to test all functionality
 */ 
class ControlPanel extends DeviceControlPanelAbstract
{
	/*
	 * NAME : ControlPanel (constructor)
	 * DESCRIPTION
	 * 		Initiate the short messages
	 * PRECONDITION
	 * 		The main instantiates MainDemo class and MainDemo instantiates this class
	 * 		in consecutively
	 * PARAMETER : none
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE : none
	 * POSTCONDITION : The control panel is ready
	 * NOTES : none
	 */ 
	public ControlPanel()
	{
		setDisplayShortMessage1("        Starting system");
		setDisplayShortMessage2("        please wait...");
	}
	
	/*
	 * NAME : button1
	 * DESCRIPTION
	 * 		set display security zone number to 1
	 * 		turn on the power LED
	 * 		display short message of what key is pressed
	 * PRECONDITION : the control panel is ready and poped up to user
	 * PARAMETER : none
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE : none
	 * POSTCONDITION : The control panel is ready and slightly changed in view
	 * NOTES : none
	 */ 
	public void button1()
	{
		// example running (key is active)
		setSecurityZoneNumber(1);
		setPoweredLED(true);
		setDisplayShortMessage1("    key 1 pressed");
		setDisplayShortMessage2("    On Button");
		// To do
		//
	}
	
	/*
	 * NAME : button2
	 * DESCRIPTION
	 * 		set display security zone number to 2
	 * 		turn off all LED and toggle messages
	 * 		display short message of what key is pressed
	 * PRECONDITION : the control panel is ready and poped up to user
	 * PARAMETER : none
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE : none
	 * POSTCONDITION : The control panel is ready and slightly changed in view
	 * NOTES : none
	 */ 
	public void button2()
	{
		// example running (key is active)
		setSecurityZoneNumber(2);
		setDisplayAway(false);
		setDisplayStay(false);
		setDisplayNotReady(false);
		setDisplayShortMessage1("    key 2 pressed");
		setDisplayShortMessage2("    Off Button");
		setArmedLED(false);
		setPoweredLED(false);
		// To do
		//
	}
	
	/*
	 * NAME : button3
	 * DESCRIPTION
	 * 		set display security zone number to 3
	 * 		turn off the armed LED and toggle messages
	 * 		display short message of what key is pressed
	 * PRECONDITION : the control panel is ready and poped up to user
	 * PARAMETER : none
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE : none
	 * POSTCONDITION : The control panel is ready and slightly changed in view
	 * NOTES : none
	 */ 
	public void button3()
	{
		// example running (key is active)
		setSecurityZoneNumber(3);
		setDisplayAway(false);
		setDisplayStay(false);
		setDisplayNotReady(false);
		setArmedLED(false);
		setDisplayShortMessage1("    key 3 pressed");
		setDisplayShortMessage2("    Reset Button");
		// To do
		//
	}
	
	/*
	 * NAME : button4
	 * DESCRIPTION
	 * 		set display security zone number to 4
	 * 		display short message of what key is pressed
	 * PRECONDITION : the control panel is ready and poped up to user
	 * PARAMETER : none
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE : none
	 * POSTCONDITION : The control panel is ready and slightly changed in view
	 * NOTES : none
	 */ 
	public void button4()
	{
		// example running (key is active)
		setSecurityZoneNumber(4);
		setDisplayShortMessage1("    key 4 pressed");
		setDisplayShortMessage2("");
		// To do
		//
	}

	/*
	 * NAME : button5
	 * DESCRIPTION
	 * 		set display security zone number to 5
	 * 		display short message of what key is pressed
	 * PRECONDITION : the control panel is ready and poped up to user
	 * PARAMETER : none
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE : none
	 * POSTCONDITION : The control panel is ready and slightly changed in view
	 * NOTES : none
	 */ 
	public void button5()
	{
		// example running (key is active)
		setSecurityZoneNumber(5);
		setDisplayShortMessage1("    key 5 pressed");
		setDisplayShortMessage2("");
		// To do
		//
	}

	/*
	 * NAME : button6
	 * DESCRIPTION
	 * 		set display security zone number to 6
	 * 		display short message of what key is pressed
	 * PRECONDITION : the control panel is ready and poped up to user
	 * PARAMETER : none
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE : none
	 * POSTCONDITION : The control panel is ready and slightly changed in view
	 * NOTES : none
	 */ 
	public void button6()
	{
		// example running (key is active)
		setSecurityZoneNumber(6);
		setDisplayShortMessage1("    key 6 pressed");
		setDisplayShortMessage2("");
		// To do
		//
	}
	
	/*
	 * NAME : button7
	 * DESCRIPTION
	 * 		set display security zone number to 7
	 * 		turn on armed LED and away
	 * 		display short message of what key is pressed
	 * PRECONDITION : the control panel is ready and poped up to user
	 * PARAMETER : none
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE : none
	 * POSTCONDITION : The control panel is ready and slightly changed in view
	 * NOTES : none
	 */ 
	public void button7()
	{
		// example running (key is active)
		setSecurityZoneNumber(7);
		setDisplayAway(true);
		setArmedLED(true);
		setDisplayShortMessage1("    key 7 pressed");
		setDisplayShortMessage2("    Away Button");
		// To do
		//
	}
	
	/*
	 * NAME : button8
	 * DESCRIPTION
	 * 		set display security zone number to 8
	 * 		turn on armed LED and stay
	 * 		display short message of what key is pressed
	 * PRECONDITION : the control panel is ready and poped up to user
	 * PARAMETER : none
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE : none
	 * POSTCONDITION : The control panel is ready and slightly changed in view
	 * NOTES : none
	 */ 
	public void button8()
	{
		// example running (key is active)
		setSecurityZoneNumber(8);
		setDisplayStay(true);
		setArmedLED(true);
		setDisplayShortMessage1("    key 8 pressed");
		setDisplayShortMessage2("    Stay Button");
		// To do
		//
	}

	/*
	 * NAME : button9
	 * DESCRIPTION
	 * 		set display security zone number to 9
	 * 		turn on not ready
	 * 		display short message of what key is pressed
	 * PRECONDITION : the control panel is ready and poped up to user
	 * PARAMETER : none
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE : none
	 * POSTCONDITION : The control panel is ready and slightly changed in view
	 * NOTES : none
	 */ 
	public void button9()
	{
		// example running (key is active)
		//JOptionPane.showMessageDialog(this, "key 9 pressed");
		setSecurityZoneNumber(9);
		setDisplayNotReady(true);
		setDisplayShortMessage1("    key 9 pressed");
		setDisplayShortMessage2("    Code Button");
		// To do
		//
	}
	
	/*
	 * NAME : buttonStar
	 * DESCRIPTION
	 * 		set display security zone number to 10
	 * 		display short message of what key is pressed
	 * PRECONDITION : the control panel is ready and poped up to user
	 * PARAMETER : none
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE : none
	 * POSTCONDITION : The control panel is ready and slightly changed in view
	 * NOTES : none
	 */ 
	public void buttonStar()
	{
		// example running (key is active)
		setSecurityZoneNumber(10);
		setDisplayShortMessage1("    key * pressed");
		setDisplayShortMessage2("    Panic Button");
		// To do
		//
	}

	/*
	 * NAME : button0
	 * DESCRIPTION
	 * 		set display security zone number to 11 
	 * 		display short message of what key is pressed
	 * PRECONDITION : the control panel is ready and poped up to user
	 * PARAMETER : none
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE : none
	 * POSTCONDITION : The control panel is ready and slightly changed in view
	 * NOTES : none
	 */ 
	public void button0()
	{
		// example running (key is active)
		setSecurityZoneNumber(11);
		setDisplayShortMessage1("    key 0 pressed");
		setDisplayShortMessage2("");
		// To do
		//
	}

	/*
	 * NAME : buttonSharp
	 * DESCRIPTION
	 * 		set display security zone number to 12
	 * 		display short message of what key is pressed
	 * PRECONDITION : the control panel is ready and poped up to user
	 * PARAMETER : none
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE : none
	 * POSTCONDITION : The control panel is ready and slightly changed in view
	 * NOTES : none
	 */ 
	public void buttonSharp()
	{
		// example running (key is active)
		setSecurityZoneNumber(12);
		setDisplayShortMessage1("    key # pressed");
		setDisplayShortMessage2("    Panic Button");
		// To do
		//
	}
	
	/*
	 * meaningless member variable in this application
	 * prevent java compiler warning message
	 */
	static final long serialVersionUID = 1943342;
}



/*
 * CLASS NAME : SensorTest 
 * DESCRIPTION
 * 		A dialog to test a set of sensors interactively 
 * 		This class maintains 5 winDoorSensors and 5 motionDetectors
 * 		The dialog shows status of sensors and provide enable/disable buttons for sensors
 * 		If the sensor state is changed asynchronously, 
 * 		the refresh button is needed to update sensor status table
 * PRECONDITION : From main dialog, "Sensor" button was chosen
 * PARAMETER : none
 * RETURN VALUE : none
 * USAGE : click a button among "reflesh", "Enable All", "Disable All" buttons
 * EXAMPLE : click reflesh button to update sensor status table
 * POSTCONDITION : none
 * NOTES : Need Sensor Test dialog to change sensor's read() value
 */ 
class SensorTest extends JFrame implements ActionListener
{	
	DeviceWinDoorSensor[] winDoorSensors;
	JTextField[] testW;	// used to display result of DeviceWinDoorSensor.test()
	JTextField[] readW;	// used to display result of DeviceWinDoorSensor.read()
		
	DeviceMotionDetector[] motionDetectors;
	JTextField[] testM;	// used to display result of DeviceMotionDetector.test()
	JTextField[] readM;	// used to display result of DeviceMotionDetector.read()
	
	static final int MAX_SENSOR = 5;	// define max number of sensors
	
	/*
	 * NAME : SensorTest (constructor)
	 * DESCRIPTION
	 * 		Initiate sensors instances
	 * 		and make layout of sensor status table and buttons
	 * PRECONDITION
	 * 		The main instantiates MainDemo class and MainDemo instantiates this class
	 * 		in consecutively
	 * PARAMETER : none
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE : none
	 * POSTCONDITION : The layout of this dialog is made
	 * NOTES : none
	 */ 
	public SensorTest()
	{
		super("Sensor Status");
		setSize(455, 570);
		getContentPane().setLayout(null);
		setResizable(false);
		
		// instanciate sensor and text field for status
		winDoorSensors = new DeviceWinDoorSensor[MAX_SENSOR];
		testW = new JTextField[MAX_SENSOR];
		readW = new JTextField[MAX_SENSOR];
		
		motionDetectors = new DeviceMotionDetector[MAX_SENSOR];
		testM = new JTextField[MAX_SENSOR];
		readM = new JTextField[MAX_SENSOR];
		
		// Loop is needed to instantiate array
		for(int i=0; i<MAX_SENSOR; i++)
		{
			winDoorSensors[i] = new DeviceWinDoorSensor();
			testW[i] = new JTextField();
			readW[i] = new JTextField();
			
			testW[i].setEditable(false);
			readW[i].setEditable(false);
			
			motionDetectors[i] = new DeviceMotionDetector();
			testM[i] = new JTextField();
			readM[i] = new JTextField();
			
			testM[i].setEditable(false);
			readM[i].setEditable(false);
		}
		
		// make buttons
		JButton buttonReflesh = new JButton("Reflesh");
		buttonReflesh.addActionListener(this);
		JButton buttonEnable = new JButton("Enable All");
		buttonEnable.addActionListener(this);
		JButton buttonDisable = new JButton("Disable All");
		buttonDisable.addActionListener(this);
		
		// attach text field to dialog
		JPanel displayPanel = new JPanel(new GridLayout(13, 3));
		displayPanel.add(new Label("Type and ID"));
		displayPanel.add(new Label("Test"));
		displayPanel.add(new Label("Read"));
		
		// attach test fields of array to dialog
		for(int i=0; i<MAX_SENSOR; i++)
		{
			displayPanel.add(new Label("WinDoor "+winDoorSensors[i].getID()));
			displayPanel.add(testW[i]);
			displayPanel.add(readW[i]);
		}
		for(int i=0; i<MAX_SENSOR; i++)
		{
			displayPanel.add(new Label("Motion "+motionDetectors[i].getID()));
			displayPanel.add(testM[i]);
			displayPanel.add(readM[i]);
		}
		
		displayPanel.add(new Label(""));
		displayPanel.add(new Label(""));
		displayPanel.add(new Label(""));
		
		// attach buttons to dialog
		displayPanel.add(buttonReflesh);
		displayPanel.add(buttonEnable);
		displayPanel.add(buttonDisable);
		
		displayPanel.setBounds(25, 25, 400, 400);
		getContentPane().add(displayPanel);
		
		// initially one time update is necessary to fill up sensor status table
		reflesh();
	}
	
	/*
	 * NAME : actionPerformed
	 * DESCRIPTION
	 * 		This function is the action listener(call back function) of three buttons
	 * 		Reflesh - simply update status information
	 * 		Enable All - call enable() of all sensors
	 * 		Disable All - call disable() of all sensors
	 * PRECONDITION : User click on any button
	 * PARAMETER : event - ActionEvent that carrying what button was pressed
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE
	 * 		If Reflesh button is clicked by user, 
	 * 		the information in sensor status table will be updated
	 * POSTCONDITION
	 * 		Maybe sensor status be changed and
	 * 		sensor status table is updated
	 * NOTES : none
	 */
	public void actionPerformed(ActionEvent event)
	{
		if(event.getActionCommand().equals("Reflesh"))
		{
		}
		else if(event.getActionCommand().equals("Enable All"))
		{
			for(int i=0; i<MAX_SENSOR; i++)
			{
				winDoorSensors[i].enable();
				motionDetectors[i].enable();
			}
		}
		else if(event.getActionCommand().equals("Disable All"))
		{
			for(int i=0; i<MAX_SENSOR; i++)
			{
				winDoorSensors[i].disable();
				motionDetectors[i].disable();
			}
		}

		reflesh();
	}
	
	/*
	 * NAME : reflesh
	 * DESCRIPTION
	 * 		This function updates the sensors status information
	 * 		call and get result from all of read() and test() methods of sensors
	 * 		display enable in red, disable in black
	 * PRECONDITION : this dialog and sensors are initiated
	 * PARAMETER : none
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE : none
	 * POSTCONDITION : sensor status table is updated
	 * NOTES : none
	 */
	void reflesh()
	{
		// loop all sensors with calling read(), test() method of sensors
		for(int i=0; i<MAX_SENSOR; i++)
		{
			if(winDoorSensors[i].test())
			{
				testW[i].setForeground(Color.RED);
				testW[i].setText("enable");
			}
			else
			{
				testW[i].setForeground(Color.BLACK);
				testW[i].setText("disable");
			}
				
			if(winDoorSensors[i].read())
			{
				readW[i].setForeground(Color.RED);
				readW[i].setText("open");
			}
			else
			{
				readW[i].setForeground(Color.BLACK);
				readW[i].setText("close");
			}

			
			if(motionDetectors[i].test())
			{
				testM[i].setForeground(Color.RED);
				testM[i].setText("enable");
			}
			else
			{
				testM[i].setForeground(Color.BLACK);
				testM[i].setText("disable");
			}
				
			if(motionDetectors[i].read())
			{
				readM[i].setForeground(Color.RED);
				readM[i].setText("detect");
			}
			else
			{
				readM[i].setForeground(Color.BLACK);
				readM[i].setText("clear");
			}
		}
	}
	
	/*
	 * meaningless member variable in this application
	 * prevent java compiler warning message
	 */
	static final long serialVersionUID = 465564;
}



/*
 * CLASS NAME : CameraTest 
 * DESCRIPTION
 * 		A dialog to test camera's functions
 * 		It has four buttons to control camera's view (pan left/right, zoom in/out)
 * 		and one big screen of camera view
 * PRECONDITION : From main dialog, "Camera" button was chosen
 * PARAMETER : none
 * RETURN VALUE : none
 * USAGE : click the buttons to pan or zoom the camera
 * EXAMPLE : click zoom in button to zoom in the camera's view
 * POSTCONDITION : none
 * NOTES : If possible, click all buttons to test all functionality
 */ 
class CameraTest extends JFrame implements ActionListener
{
	CameraView cameraView = new CameraView();
	
	/*
	 * NAME : CameraTest (constructor)
	 * DESCRIPTION
	 * 		Initiate sensors instances
	 * 		and place buttons(pan left/right, zoom in/out) and camera screen
	 * PRECONDITION
	 * 		The main instantiates MainDemo class and MainDemo instantiates this class
	 * 		in consecutively
	 * PARAMETER : none
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE : none
	 * POSTCONDITION : The layout of this dialog is made
	 * NOTES : none
	 */ 
	public CameraTest()
	{
		super("Camera Example");
		setSize(455, 570);
		getContentPane().setLayout(null);
		setResizable(false);
		
		// make buttons
		JButton buttonPanLeft = new JButton("PanLeft");
		buttonPanLeft.addActionListener(this);
		JButton buttonPanRight = new JButton("PanRight");
		buttonPanRight.addActionListener(this);
		JButton buttonZoomIn = new JButton("ZoomIn");
		buttonZoomIn.addActionListener(this);
		JButton buttonZoomOut = new JButton("ZoomOut");
		buttonZoomOut.addActionListener(this);
		
		// attach buttons to dialog
		JPanel displayPanel = new JPanel(new GridLayout(2, 2));
		displayPanel.add(buttonPanLeft);
		displayPanel.add(buttonPanRight);
		displayPanel.add(buttonZoomIn);
		displayPanel.add(buttonZoomOut);
		displayPanel.setBounds(25, 25, 400, 60);
		getContentPane().add(displayPanel);
		
		// attach camera view to dialog
		displayPanel = new JPanel(new GridLayout(1, 1));
		displayPanel.add(cameraView);
		displayPanel.setBounds(25, 110, 400, 400);
		getContentPane().add(displayPanel);
	}
	
	/*
	 * NAME : actionPerformed
	 * DESCRIPTION
	 * 		This function is the action listener(call back function) of four buttons
	 * 		PanLeft, PanRight, ZoomIn, ZoomOut to control camera view
	 * PRECONDITION : User click on any button
	 * PARAMETER : event - ActionEvent that carrying what button was pressed
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE
	 * 		If PanLeft button is clicked by user, 
	 * 		the view of camera is panned left
	 * POSTCONDITION : View of camera is changed and display is repainted.
	 * NOTES : none
	 */
	public void actionPerformed(ActionEvent event)
	{
		if(event.getActionCommand().equals("PanLeft"))
		{
			cameraView.camera.panLeft();
		}
		else if(event.getActionCommand().equals("PanRight"))
		{
			cameraView.camera.panRight();
		}
		else if(event.getActionCommand().equals("ZoomIn"))
		{
			cameraView.camera.zoomIn();	
		}
		else if(event.getActionCommand().equals("ZoomOut"))
		{
			cameraView.camera.zoomOut();	
		}
		
		cameraView.repaint();
	}
	
	/*
	 * meaningless member variable in this application
	 * prevent java compiler warning message
	 */
	static final long serialVersionUID = 99767;
}


/*
 * CLASS NAME : CameraView 
 * DESCRIPTION
 * 		A simple java component that hold the BufferedImage from DeviceCamera
 *		and it has own drawing function 
 * PRECONDITION
 * 		This component was attached to any JFrame
 * 		There exist camera1.jpg file
 * PARAMETER : none
 * RETURN VALUE : none
 * USAGE : call repaint() method to reflesh the camera view
 * EXAMPLE : none
 * POSTCONDITION : An image from a camera is displaying by this component
 * NOTES
 * 		In constructor, changing integer parameter of camera.setID(int)
 * 		can change loading image file - camera<int>.jpg  
 */ 
class CameraView extends Component implements Runnable
{
	// Real instance of the Camera
	DeviceCamera camera = new DeviceCamera();
	
	/*
	 * NAME : CameraView (constructor)
	 * DESCRIPTION
	 * 		Load image file to DeviceCamera
	 * 		and draw imagee to attached dialog
	 * 		and initiate Thread to periodic update of camera view
	 * PRECONDITION
	 * 		The main instantiates MainDemo class and MainDemo instantiates CameraTest class
	 * 		and CameraTest instantiates this class in consecutively
	 * PARAMETER : none
	 * RETURN VALUE : none
	 * USAGE : none
	 * EXAMPLE : none
	 * POSTCONDITION
	 * 		JPG image file is loaded to memory and ready to draw
	 * 		and thread is running to update camera view
	 * NOTES : none
	 */ 
	public CameraView()
	{
		camera.setID(2);
		
		Thread th = new Thread(this);
		th.start();
	}
	
	/*
	 * NAME : paint
	 * DESCRIPTION
	 * 		This function get new img from camera device and re-draw image to attached dialog
	 * PRECONDITION
	 * 		JPG image file is loaded to camera device's memory and ready to draw
	 * PARAMETER : g - Graphics to draw image to display
	 * RETURN VALUE : none
	 * USAGE : To draw image, should know about usage of Graphic class 
	 * EXAMPLE : none
	 * POSTCONDITION : User can see the image from the camera
	 * NOTES : none
	 */ 
    public void paint(Graphics g) 
    {
        // Draw the image stretched to exactly cover the size of the drawing area.
        Dimension size = getSize();
        BufferedImage img = camera.getView();
        
        g.drawImage(img, 
                    0, 0, size.width, size.height,
                    0, 0, img.getWidth(null), img.getHeight(null),
                    null);                    
    }
     
    /*
	 * NAME : run
	 * DESCRIPTION
	 * 		This function executed by thread concurrently
	 *		and updates view of the camera at periodic interval
	 * PRECONDITION
	 * 		JPG image file is loaded to camera device's memory and ready to draw
	 * PARAMETER : g - Graphics to draw image to display
	 * RETURN VALUE : none
	 * USAGE : none 
	 * EXAMPLE : none
	 * POSTCONDITION : User can see the image from the camera
	 * NOTES
	 * 		Sleep interval should be less than 1000
	 * 		This function should have infinte loop
	 */ 
	public void run()
	{
		while(true)
		{
			try
			{
				Thread.sleep(100);
			}
			catch(InterruptedException e)
			{
			}
			repaint();
		}
	}
    
    /*
	 * meaningless member variable in this application
	 * prevent java compiler warning message
	 */
	static final long serialVersionUID = 45634;
}


